@@ -250,7 +250,7 @@ typedef struct dsc$descriptor   DESC;
 #endif
 
 #ifndef XDG_VIMRC_FILE
-# define XDG_VIMRC_FILE (mch_getenv("XDG_CONFIG_HOME") \
+# define XDG_VIMRC_FILE (mch_getenv((char_u *)"XDG_CONFIG_HOME") \
 	? "$XDG_CONFIG_HOME/vim/vimrc" \
 	: "~/.config/vim/vimrc")
 #endif
